import json
import uuid

import requests

from .metistypes import (
    Message,
    MessageContent,
    MessageRequest,
    MessageStream,
    Session,
    SessionRequest,
    SessionUser,
    Task,
    TaskResult,
)


class MetisBot:
    API_V1_SESSION = "https://api.metisai.ir/api/v1/chat/session"

    def __init__(self, api_key, bot_id):
        self.api_key = api_key
        self.endpoints = {
            "sessions": f"{self.API_V1_SESSION}s?userId={{user_id}}",
            "session": f"{self.API_V1_SESSION}",
            "get_session": f"{self.API_V1_SESSION}/{{session_id}}",
            "message": f"{self.API_V1_SESSION}/{{session_id}}/message",
            "async_task": f"{self.API_V1_SESSION}/{{session_id}}/message/async",
            "get_async_task": f"{self.API_V1_SESSION}/{{session_id}}/message/async/{{task_id}}",
            "stream": f"{self.API_V1_SESSION}/{{session_id}}/message/stream",
        }
        self.headers = {
            "Content-Type": "application/json",
            "X-Api-Key": self.api_key,
        }
        self.bot_id = bot_id

    def create_session(self, user_id: str = None) -> Session:
        if user_id is None:
            user_id = str(uuid.uuid4())

        session_request = SessionRequest(
            botId=self.bot_id, user=SessionUser(id=user_id, name="_")
        )

        response = requests.post(
            url=self.endpoints.get("session"),
            headers=self.headers,
            json=session_request.model_dump(),
        )
        response.raise_for_status()
        return Session(**response.json())

    def list_sessions(self, user_id: str) -> list[Session]:
        url = self.endpoints.get("sessions").format(user_id=user_id)
        response = requests.get(url, headers=self.headers)
        response.raise_for_status()
        return [Session(**data) for data in response.json()]

    def retrieve_session(self, session_id: str) -> Session:
        url = self.endpoints.get("get_session").format(session_id=session_id)
        response = requests.get(url, headers=self.headers)
        response.raise_for_status()
        return Session(**response.json())

    def delete_session(self, session: Session) -> None:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        url = self.endpoints.get("get_session").format(session_id=session_id)
        response = requests.delete(url, headers=self.headers)
        response.raise_for_status()

    def send_message(self, session: Session, prompt: str) -> Message:
        if isinstance(session, (str, uuid.UUID)):
            pass
        else:
            session.id

        url = self.endpoints.get("message").format(session_id=session.id)
        data = MessageRequest(
            message=MessageContent(
                type="USER",
                content=prompt,
            )
        )
        response = requests.post(url, headers=self.headers, json=data.model_dump())
        response.raise_for_status()
        return Message(**response.json())

    def send_message_async(self, session: Session, prompt: str) -> Task:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        url = self.endpoints.get("async_task").format(session_id=session_id)
        data = MessageRequest(
            message=MessageContent(
                type="USER",
                content=prompt,
            )
        )
        response = requests.post(url, headers=self.headers, json=data.model_dump())
        response.raise_for_status()
        return Task(**response.json())

    def retrieve_async_task(self, session: Session, task: Task) -> TaskResult:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        if isinstance(task, (str, uuid.UUID)):
            task_id = task
        else:
            task_id = task.taskId

        url = self.endpoints.get("get_async_task").format(
            session_id=session_id, task_id=task_id
        )
        response = requests.get(url, headers=self.headers)
        response.raise_for_status()
        return TaskResult(**response.json())

    def stream_messages(
        self, session: Session, prompt: str, split_criteria: dict = None
    ):
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session_id

        url = self.endpoints.get("stream").format(session_id=session_id)
        data = MessageRequest(
            message=MessageContent(
                type="USER",
                content=prompt,
            )
        )
        response = requests.post(
            url, headers=self.headers, json=data.model_dump(), stream=True
        )
        response.raise_for_status()

        if split_criteria.get("words"):
            split_criteria["splitter"] = " "
        elif split_criteria.get("sentence"):
            split_criteria["splitter"] = ".?!:"
        elif split_criteria.get("line"):
            split_criteria["splitter"] = "\n"

        buffer = ""
        for line in response.iter_lines():
            if line:
                data = line.decode("utf-8").replace("data:", "")
                msg = MessageStream(**json.loads(data))
                if not split_criteria:
                    yield MessageStream(**json.loads(data))
                    continue

                buffer += msg.message.content
                if split_criteria.get("min-length"):
                    if len(buffer) >= split_criteria.get("min-length"):
                        yield MessageStream(
                            message=MessageContent(
                                type="AI", content=buffer, attachments=None
                            )
                        )
                        buffer = ""

                if split_criteria.get("splitter"):
                    for splitter in split_criteria.get("splitter"):
                        if splitter in buffer:
                            yield MessageStream(
                                message=MessageContent(
                                    type="AI", content=buffer, attachments=None
                                )
                            )
                            buffer = ""

        yield MessageStream(
            message=MessageContent(type="AI", content=buffer, attachments=None)
        )



from datetime import datetime
from typing import Any, Literal

from pydantic import BaseModel


class User(BaseModel):
    id: str
    username: str  # Email
    firstName: str
    lastName: str
    activated: bool = False


class APIKey(BaseModel):
    id: str
    apiKey: str
    userId: str
    # description: str


class Provider(BaseModel):
    name: str
    model: str


class ProviderConfig(BaseModel):
    provider: Provider
    args: dict[str, Any]


class ChatProvider(BaseModel):
    name: str
    model: str
    currency: str
    fixedCallPrice: float
    inputTokenUnitPrice: float
    outputTokenUnitPrice: float


class Summarizer(BaseModel):
    provider: Provider
    args: dict[str, Any]


class BotRequest(BaseModel):
    name: str
    instructions: str
    providerConfig: ProviderConfig
    corpusIds: list[str]
    summarizer: Summarizer
    functions: list[str]
    description: str
    avatar: str
    enabled: bool


class Bot(BaseModel):
    id: str
    name: str
    userId: str
    instructions: str | None = None
    externalId: str
    providerConfig: ProviderConfig
    chatProvider: ChatProvider
    corpora: list[
        str
    ]  # Adjust the type if the corpora list contains more complex objects.
    summarizer: Summarizer
    functions: list[str]  # Adjust if functions contain more complex objects.
    description: str | None = None
    avatar: str | None = None
    enabled: bool = False


class Attachment(BaseModel):
    content: str
    contentType: str


class MessageContent(BaseModel):
    type: Literal["USER", "AI"]
    content: str
    attachments: list[Attachment] | None = None


class ExternalUser(BaseModel):
    id: str
    name: str


class BotInfo(BaseModel):
    botId: str
    sessionId: str


class MessageRequest(BaseModel):
    message: MessageContent


class Message(BaseModel):
    id: str
    type: Literal["USER", "AI"]
    content: str | None = None
    attachments: list[Attachment] | None = None
    timestamp: datetime
    finishReason: Literal["STOP", "LENGTH", "CONTENT_FILTER"] | None


class MessageStream(BaseModel):
    message: MessageContent


class Task(BaseModel):
    taskId: str


class TaskResult(BaseModel):
    status: Literal["RUNNING", "FINISHED", "FAILED"]
    message: Message | None = None
    percentage: int | None = None


class SessionUser(BaseModel):
    id: str | None = None
    name: str | None = None


class SessionRequest(BaseModel):
    botId: str | None = None
    user: SessionUser | None = None
    initialMessages: list[MessageContent] = []


class Session(BaseModel):
    id: str
    botId: str
    user: SessionUser
    messages: list[Message]
    startDate: datetime



import json
import uuid

import aiohttp

from ..metistypes import (
    Attachment,
    Message,
    MessageContent,
    MessageRequest,
    MessageStream,
    Session,
    SessionRequest,
    SessionUser,
    Task,
    TaskResult,
)


class AsyncMetisBot:
    API_V1_SESSION = "https://api.metisai.ir/api/v1/chat/session"

    def __init__(self, api_key, bot_id):
        self.api_key = api_key
        self.endpoints = {
            "sessions": f"{self.API_V1_SESSION}s?userId={{user_id}}",
            "session": f"{self.API_V1_SESSION}",
            "get_session": f"{self.API_V1_SESSION}/{{session_id}}",
            "message": f"{self.API_V1_SESSION}/{{session_id}}/message",
            "async_task": f"{self.API_V1_SESSION}/{{session_id}}/message/async",
            "get_async_task": f"{self.API_V1_SESSION}/{{session_id}}/message/async/{{task_id}}",
            "stream": f"{self.API_V1_SESSION}/{{session_id}}/message/stream",
        }
        self.headers = {
            "Content-Type": "application/json",
            "X-Api-Key": self.api_key,
        }
        self.bot_id = bot_id

    async def _request(self, method: str, endpoint: str, **kwargs):
        url = self.endpoints.get(endpoint).format(**kwargs.pop("url_params", {}))
        async with aiohttp.ClientSession() as session:
            async with session.request(
                method, url, headers=self.headers, **kwargs
            ) as response:
                response.raise_for_status()
                return await response.json()

    async def create_session(self, user_id: str = None) -> Session:
        if user_id is None:
            user_id = str(uuid.uuid4())

        session_request = SessionRequest(
            botId=self.bot_id, user=SessionUser(id=user_id, name="_")
        )

        response_data = await self._request(
            method="POST",
            endpoint="session",
            json=session_request.model_dump(),
        )
        return Session(**response_data)

    async def list_sessions(self, user_id: str) -> list[Session]:
        response_data = await self._request(
            method="GET", endpoint="sessions", url_params={"user_id": user_id}
        )
        return [Session(**data) for data in response_data]

    async def retrieve_session(self, session_id: str) -> Session:
        response_data = await self._request(
            method="GET",
            endpoint="get_session",
            url_params={"session_id": session_id},
        )
        return Session(**response_data)

    async def delete_session(self, session: Session) -> None:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id
        await self._request(
            method="DELETE",
            endpoint="get_session",
            url_params={"session_id": session_id},
        )

    async def send_message(self, session: Session, prompt: str) -> Message:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        data = MessageRequest(
            message=MessageContent(
                type="USER",
                content=prompt,
            )
        )
        response_data = await self._request(
            method="POST",
            endpoint="message",
            url_params={"session_id": session_id},
            json=data.model_dump(),
        )
        return Message(**response_data)

    async def send_message_with_attachment(self, session: Session, prompt: str, attachment_url: str, attachment_type: str = "IMAGE") -> Message:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        data = MessageRequest(
            message=MessageContent(
                type="USER",
                content=prompt,
                attachments=[
                    Attachment(content=attachment_url, contentType=attachment_type)
                ],
            )
        )
        response_data = await self._request(
            method="POST",
            endpoint="message",
            url_params={"session_id": session_id},
            json=data.model_dump(),
        )
        return Message(**response_data)

    async def send_message_async(self, session: Session, prompt: str) -> Task:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        data = MessageRequest(
            message=MessageContent(
                type="USER",
                content=prompt,
            )
        )
        response_data = await self._request(
            method="POST",
            endpoint="async_task",
            url_params={"session_id": session_id},
            json=data.model_dump(),
        )
        return Task(**response_data)

    async def retrieve_async_task(self, session: Session, task: Task) -> TaskResult:
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        if isinstance(task, (str, uuid.UUID)):
            task_id = task
        else:
            task_id = task.taskId

        response_data = await self._request(
            method="GET",
            endpoint="get_async_task",
            url_params={"session_id": session_id, "task_id": task_id},
        )
        return TaskResult(**response_data)

    async def stream_messages(
        self, session: Session, prompt: str, split_criteria: dict = None
    ):
        if isinstance(session, (str, uuid.UUID)):
            session_id = session
        else:
            session_id = session.id

        url = self.endpoints.get("stream").format(session_id=session_id)
        data = MessageRequest(
            message=MessageContent(
                type="USER",
                content=prompt,
            )
        )

        async with aiohttp.ClientSession() as _session:
            async with _session.post(
                url, headers=self.headers, json=data.model_dump()
            ) as response:
                response.raise_for_status()
                buffer = ""

                async for line in response.content:
                    if line:
                        data = line.decode("utf-8").strip("data:").strip()
                        if not data:
                            continue
                        msg = MessageStream(**json.loads(data))
                        if not split_criteria:
                            yield msg
                            continue

                        buffer += msg.message.content
                        if split_criteria.get("min-length") and len(
                            buffer
                        ) >= split_criteria.get("min-length"):
                            yield MessageStream(
                                message=MessageContent(
                                    type="AI", content=buffer, attachments=None
                                )
                            )
                            buffer = ""

                        if split_criteria.get("splitter"):
                            for splitter in split_criteria.get("splitter"):
                                if splitter in buffer:
                                    yield MessageStream(
                                        message=MessageContent(
                                            type="AI",
                                            content=buffer,
                                            attachments=None,
                                        )
                                    )
                                    buffer = ""

                yield MessageStream(
                    message=MessageContent(type="AI", content=buffer, attachments=None)
                )

    async def close(self):
        if self._session is not None:
            await self._session.close()
            self._session = None



import asyncio
import os
import unittest

from metisai.async_metis import AsyncMetisBot


class TestAsyncTapSageBot(unittest.IsolatedAsyncioTestCase):
    @classmethod
    def setUpClass(cls):
        api_key = os.getenv("METIS_API_KEY")
        bot_id = os.getenv("METIS_BOT_ID")
        cls.metis_bot = AsyncMetisBot(api_key, bot_id)
        cls.prompt = os.getenv("PROMPT")

    @staticmethod
    async def async_test(test_coro):
        loop = asyncio.get_event_loop()
        return await loop.run_until_complete(test_coro)

    async def test_create_session(self):
        print("test_create_session")
        session = await self.metis_bot.create_session()
        self.assertIsNotNone(session.id)
        print(session)
        await self.metis_bot.delete_session(session)

    async def test_send_message(self):
        print("test_send_message")
        session = await self.metis_bot.create_session()
        message = await self.metis_bot.send_message(session, self.prompt)
        self.assertIsInstance(message.content, str)
        print(message.content)
        await self.metis_bot.delete_session(session)

    async def test_send_message_async(self):
        print("test_send_message_async")
        session = await self.metis_bot.create_session()
        task = await self.metis_bot.send_message_async(session, self.prompt)

        while True:
            task_result = await self.metis_bot.retrieve_async_task(session, task)
            if task_result.status == "FINISHED":
                break
            await asyncio.sleep(1)
        self.assertIsInstance(task_result.message.content, str)
        print(task_result.message.content)
        await self.metis_bot.delete_session(session)

    async def test_stream_messages(self):
        print("test_stream_messages")
        session = await self.metis_bot.create_session()
        async for message in self.metis_bot.stream_messages(
            session, self.prompt, split_criteria={"line": True}
        ):
            print(message.message.content)
        await self.metis_bot.delete_session(session)

    def test_all_async(self):
        loop = asyncio.get_event_loop()
        loop.run_until_complete(self.test_create_session())
        loop.run_until_complete(self.test_send_message())
        loop.run_until_complete(self.test_send_message_async())
        loop.run_until_complete(self.test_stream_messages())


if __name__ == "__main__":
    unittest.main()
